#ifndef MP4V2_PROJECT_H
#define MP4V2_PROJECT_H

/*****************************************************************************/

#define MP4V2_PROJECT_name            "MP4v2"
#define MP4V2_PROJECT_name_lower      "mp4v2"
#define MP4V2_PROJECT_name_upper      "MP4V2"
#define MP4V2_PROJECT_name_formal     "MP4v2 2.1.3"
#define MP4V2_PROJECT_url_website     "https://mp4v2.org"
#define MP4V2_PROJECT_url_downloads   "https://github.com/enzo1982/releases"
#define MP4V2_PROJECT_url_discussion  "https://github.com/enzo1982/discussions"
#define MP4V2_PROJECT_url_bugreport   "https://github.com/enzo1982/issues"
#define MP4V2_PROJECT_support         "<support@mp4v2.org>"
#define MP4V2_PROJECT_version         "2.1.3"
#define MP4V2_PROJECT_version_hex     0x00020103
#define MP4V2_PROJECT_version_major   2
#define MP4V2_PROJECT_version_minor   1
#define MP4V2_PROJECT_version_point   3
#define MP4V2_PROJECT_repo_url        "git@github.com:enzo1982/mp4v2.git"
#define MP4V2_PROJECT_build           "Mon Mar 23 09:34:02 UTC 2026"

/*****************************************************************************/

#endif /* MP4V2_PROJECT_H */
